#!/usr/bin/perl -w

# Copyright 2013, 2014, 2015, 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.004;
use strict;
use List::Util 'max','min','sum';
use Math::Libm 'hypot';
use Math::BaseCnv 'cnv';

use FindBin;
use lib::abs "$FindBin::Bin/../tools";

# uncomment this to run the ### lines
# use Smart::Comments;

{
  # X axis points

  # X
  # S -> S,S,E,N,W
  # E -> S,S,E,N,N
  # N -> W,S,E,N,N
  # W -> W,N,E,S,W

  require MyFSM;
  my $table = { S => {4=>'S',3=>'S',2=>'E',1=>'N',0=>'W'},
                E => {4=>'S',3=>'S',2=>'E',1=>'N',0=>'N'},
                N => {4=>'W',3=>'S',2=>'E',1=>'N',0=>'N'},
                W => {4=>'W',3=>'S',2=>'E',1=>'N',0=>'W'} };
  my $fsm = MyFSM->new
    (table => $table,
     initial => 'W',
     accepting => { S => 1 },
    );
  $fsm->simplify;
  $fsm->show;
  $fsm->view;

  $fsm = $fsm->reverse;
  $fsm->simplify;
  $fsm->show;
  $fsm->view;

  exit 0;
}

{
  # X axis points

  # X
  # S -> S,S,E,N,W
  # E -> S,S,E,N,N
  # N -> W,S,E,N,N
  # W -> W,N,E,S,W

  require MyFSM;
  my $table = { S => {0=>'S',1=>'S',2=>'E',3=>'N',4=>'W'},
                E => {0=>'S',1=>'S',2=>'E',3=>'N',4=>'N'},
                N => {0=>'W',1=>'S',2=>'E',3=>'N',4=>'N'},
                W => {0=>'W',1=>'S',2=>'E',3=>'N',4=>'W'} };
  my $fsm = MyFSM->new
    (table => $table,
     initial => 'S',
     accepting => { S => 1 },
    );
  $fsm->simplify;
  $fsm->show;
  $fsm->view;

  $fsm = $fsm->reverse;
  $fsm->simplify;
  $fsm->show;
  $fsm->view;

  exit 0;
}
