#!/usr/bin/perl -w

# Copyright 2014, 2016, 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use List::Util 'min','max';
use Math::BigRat;
use Math::PlanePath::R5DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib "$FindBin::Bin/../tools";
use MyR5Dragon;
use MyLineSegments;
use MyPrintwrap ':all';

# uncomment this to run the ### lines
# use Smart::Comments;


printwrap("  % generated by tools/$FindBin::Script\n");

my $level = 2;

my $path = Math::PlanePath::R5DragonCurve->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);
my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);
my ($mx,$my) = ($x_hi/2, $y_hi/2);

my $segments = MyLineSegments->new;
foreach my $n ($n_lo .. $n_hi-1) {
  $segments->add($path->n_to_xy($n),
                 $path->n_to_xy($n+1));
}
$Printwrap = 2;
printwrap("\\draw[my line]");
$Printwrap += 2;
foreach my $line ($segments->lines) {
  my ($x1,$y1, $x2,$y2) = @$line;
  printwrap(" ($x1,$y1)--($x2,$y2)");
}
printwrap(";\n");

my $inertia = 0;
$Printwrap = 2;
foreach my $n ($n_lo .. $n_hi-1) {
  my ($x,$y) = $path->n_to_xy($n);
  my ($dx,$dy) = $path->n_to_dxdy($n);

  my ($sx,$sy) = ($x+$dx/2, $y+$dy/2);     # midpoint of segment
  my $name = ($n == ($n_hi-1)/2 ? ' (middle)' : '');
  printwrap(" \\node$name at ($sx,$sy) [my dot];");

  my ($rx,$ry) = ($sx-$mx, $sy-$my);  # radial dist from centre
  $inertia += $rx*$rx + $ry*$ry;
}
printwrap_newline();

$inertia = Math::BigRat->new($inertia) / 5**$level;
my $inertia_num = $inertia->numerator;
my $inertia_den = $inertia->denominator;

print "  \\newcommand{\\MyLevel}{$level}\n";
print "  \\newcommand{\\MyInertia}{\\tfrac{$inertia_num}{$inertia_den}}\n";
print "% GP-Test  Iz($level)==$inertia\n";

print "  % end generated\n";
exit 0;
