#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.004;
use strict;
use Math::PlanePath::R5DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyR5Dragon ':all';
use MyLineSegments;
use MyPrintwrap ':all';

# uncomment this to run the ### lines
# use Smart::Comments;


my $level = 2;
print "  % generated by tools/$FindBin::Script\n";
print "  \\newcommand{\\MyLevel}{$level}\n";

my $path = Math::PlanePath::R5DragonCurve->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);
my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);

my $segments = MyLineSegments->new;
foreach my $n ($n_lo .. $n_hi - 1) {
  $segments->add($path->n_to_xy($n),
                 $path->n_to_xy($n+1));
}
$Printwrap = 2;
printwrap("% N=$n_lo to $n_hi\n");
printwrap("\\fill (0,0) circle;");
printwrap(" \\fill ($x_hi,$y_hi) circle;\n");
printwrap("\\draw ");
$Printwrap += 2;
$segments->printwrap;
printwrap(";\n");

my %count;
foreach my $side ('right','left') {
  my $colour = $side eq 'right' ? '' : '[my left colour]';
  $Printwrap = 2;
  printwrap("\\fill$colour foreach \\p in {");
  $Printwrap += 2;
  my %seen;
  foreach my $n ($n_lo .. $n_hi - 1) {
    my ($x1,$y1) = $path->n_to_xy($n);
    my ($dx,$dy) = $path->n_to_dxdy($n);
    if ($side eq 'right') {
      $path->_UNDOCUMENTED__n_segment_is_right_boundary($n) or next;
    } else {
      $path->_UNDOCUMENTED__n_segment_is_right_boundary($n_hi-1 - $n) or next;
      ### left boundary ...
      $x1 += $dx;
      $y1 += $dy;
      $dx = -$dx;
      $dy = -$dy;
    }
    my $x2 = $x1 + $dx + $dy;  # opposite corner
    my $y2 = $y1 + $dy - $dx;
    if ($x1 > $x2) {
      ($x1,$x2) = ($x2,$x1);
    }
    if ($y1 > $y2) {
      ($y1,$y2) = ($y2,$y1);
    }
    next if $seen{"$x1,$y1"}++;
    printwrap("($x1,$y1),");
    $count{$side}++;
  }
  printwrap("}");
  printwrap("{ \\p +(\\MyGap,\\MyGap) rectangle +(1-\\MyGap,1-\\MyGap) };");
}
my $count = $count{'left'} + $count{'right'};

printwrap_newline();
print "  \\newcommand{\\MyBQ}{$count}\n";
print "  \\newcommand{\\MyRQ}{$count{right}}\n";
print "  \\newcommand{\\MyLQ}{$count{left}}\n";
print "% GP-Test-Last  BQ($level)==$count\n";
print "% GP-Test-Last  RQ($level)==$count{right}\n";
print "% GP-Test-Last  LQ($level)==$count{left}\n";

print "  % end generated\n";
exit 0;
