#!/usr/bin/perl -w

# Copyright 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;

use FindBin;
use lib $FindBin::Bin;
use MyR5Dragon;
use MyGraphs;


my $max_level = 3;
if (@ARGV) { $max_level = shift @ARGV; }

print "% generated by tools/$FindBin::Script $max_level\n";

my $var = 'gen';

my %arrays;
foreach my $k (0 .. $max_level) {
  my $graph = MyR5Dragon::r5join_area_tree_Graph($k);
  my $apsp = $graph->APSP_Floyd_Warshall;
  my $num_vertices = $graph->vertices;
  my $JAdiameter = $apsp->diameter || 0;
  my %seen;
  my $JAdiameterCount = 0;
  my %JAdiameterEnds;
  my %JAdiameterVertices;
  if ($num_vertices == 1) {
    my @vertices = $graph->vertices;
    $JAdiameterEnds{$vertices[0]} = 1;
    $JAdiameterVertices{$vertices[0]} = 1;
  }
  $apsp->for_shortest_paths
    (sub {
       my ($t, $u,$v, $n) = @_;
       my $len = $t->path_length($u,$v);
       if ($len == $JAdiameter && ! $seen{$u}->{$v}) {
         $JAdiameterCount++;
         $seen{$u}->{$v} = 1;
         $seen{$v}->{$u} = 1;
         $JAdiameterEnds{$u} = 1;
         $JAdiameterEnds{$v} = 1;
         if ($k <= 4) {
           @JAdiameterVertices{$t->path_vertices($u,$v)} = ();  # hash slice
         }
       }
     });
  push @{$arrays{'JAdiameter'}},      $JAdiameter;
  push @{$arrays{'JAdiameterCount'}}, $JAdiameterCount;
  push @{$arrays{'JAdiameterEnds'}},  scalar(keys %JAdiameterEnds);
  if ($k <= 4) {
    push @{$arrays{'JAdiameterVertices'}},  scalar(keys %JAdiameterVertices);
  }
}

foreach my $name ('JAdiameter','JAdiameterCount','JAdiameterEnds','JAdiameterVertices',
                 ) {
  my $aref = $arrays{$name};
  print "% GP-DEFINE  ${var}_${name}_samples = [", join(',',@$aref),"];\n";
}

print "% end generated\n";
exit 0;
