#!/usr/bin/perl -w

# Copyright 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use Math::PlanePath::QuintetCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyLineSegments;
use MyPrintwrap ':all';


my $level = 3;
my $arms = 4;

print "  % generated by tools/$FindBin::Script\n";
my $path = Math::PlanePath::QuintetCurve->new (arms => $arms);

my ($n_lo, $n_hi) = $path->level_to_n_range($level);

$Printwrap = 2;
printwrap("\\node (start) at (0,0) [my dot];\n");
foreach my $offset (0 .. $arms-1) {
  my ($x_hi,$y_hi) = $path->n_to_xy($n_hi - $offset);
  my $arm = -$offset % 4;
  printwrap("\\node (end-$arm) at ($x_hi,$y_hi) [my dot];\n");
}

my $segments = MyLineSegments->new;
$segments->add(0,0, 0,1);
$segments->add(0,0, -1,0);
$segments->add(0,0, 0,-1);
foreach my $n ($n_lo .. $n_hi - $arms) {
  my ($x1,$y1) = $path->n_to_xy($n);
  my ($x2,$y2) = $path->n_to_xy($n + $arms);
  $segments->add($x1,$y1, $x2,$y2);
}

$Printwrap = 2;
printwrap("\\draw ");
$Printwrap += 2;
$segments->printwrap;
printwrap(";");
my $count = $segments->lines_joined_count_points;
printwrap(" % $count points\n");

print "  % end generated\n";
exit 0;
