#!/usr/bin/perl -w

# Copyright 2017, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  See the file COPYING.  If not, see
# <http://www.gnu.org/licenses/>.


# Usage: ./my-check-tex-date.pl filename.tex...
#
# Check that the \date{} shown in each filename.tex is current.
#

use 5.010;
use strict;
use warnings;
use File::Slurp;
use POSIX ();

# uncomment this to run the ### lines
# use Smart::Comments;


my $want_date = POSIX::strftime('%B %Y', localtime(time()));

# return the line number of pos($str) within $str
# the first line is 1
sub pos_linenum {
  my ($str) = @_;
  $str = substr($str, 0, pos($_[0]));
  return 1 + scalar($str =~ tr/\n//);
}

my $good = 1;
foreach my $filename (@ARGV) {
  my $str = File::Slurp::slurp($filename);

  $str =~ /\\date\{(.*?)\}/g or die;
  my $whole_date = $1;
  $whole_date =~ s/\\kern[0-9.]+em//g;
  my $found = 0;
  foreach my $got_date (split /,\s*/, $whole_date) {
    ### $got_date
    if ($got_date eq $want_date) {
      $found = 1;
      last;
    }
  }
  if (! $found) {
    my $linenum = pos_linenum($str);
    print "$filename:$linenum: \\date got \"$whole_date\" want somewhere \"$want_date\"\n";
    $good = 0;
  }

  if ($str =~ /%.*\\usepackage\{breakurl\}/m) {
    my $linenum = pos_linenum($str);
    print "$filename:$linenum: \\usepackage{breakurl} commented out\n";
    $good = 0;
  }
}
exit ($good ? 0 : 1);
